from typing import AbstractSet, Sequence, Tuple

from ..alignment import AlignmentModelType, AlignmentModel

class OnlineTrainingParameters:
    algorithm: int
    learning_rate_policy: int
    learn_step_size: float
    em_iters: int
    e: int
    r: int

    def __init__(self) -> None: ...

class SmtModel:
    def __init__(self, model_type: AlignmentModelType) -> None: ...
    @property
    def non_monotonicity(self) -> int: ...
    @non_monotonicity.setter
    def non_monotonicity(self, value: int) -> None: ...
    @property
    def w(self) -> float: ...
    @w.setter
    def w(self, value: float) -> None: ...
    @property
    def a(self) -> int: ...
    @a.setter
    def a(self, value: int) -> None: ...
    @property
    def e(self) -> int: ...
    @e.setter
    def e(self, value: int) -> None: ...
    @property
    def heuristic(self) -> int: ...
    @heuristic.setter
    def heuristic(self, value: int) -> None: ...
    @property
    def online_training_parameters(self) -> OnlineTrainingParameters: ...
    @online_training_parameters.setter
    def online_training_parameters(self, value: OnlineTrainingParameters) -> None: ...
    @property
    def weights(self) -> Sequence[float]: ...
    @weights.setter
    def weights(self, value: Sequence[float]) -> None: ...
    @property
    def direct_word_alignment_model(self) -> AlignmentModel: ...
    @property
    def inverse_word_alignment_model(self) -> AlignmentModel: ...
    def load_translation_model(self, prefix_filename: str) -> bool: ...
    def load_language_model(self, prefix_filename: str) -> bool: ...
    def print_translation_model(self, prefix_filename: str) -> bool: ...
    def print_language_model(self, prefix_filename: str) -> bool: ...
    def clear(self) -> None: ...

class TranslationData:
    target: Sequence[str]
    source_segmentation: Sequence[Tuple[int, int]]
    target_segment_cuts: Sequence[int]
    target_unknown_words: AbstractSet[int]
    score: float
    score_components: Sequence[float]

class WordGraphArc:
    in_state: int
    out_state: int
    score: float
    words: Sequence[str]
    source_start_index: int
    source_end_index: int
    is_unknown: bool

class WordGraphState:
    in_arc_ids: Sequence[int]
    out_arc_ids: Sequence[int]

class WordGraph:
    @property
    def empty(self) -> bool: ...
    @property
    def num_arcs(self) -> int: ...
    @property
    def num_states(self) -> int: ...
    @property
    def initial_state_score(self) -> float: ...
    @property
    def final_states(self) -> AbstractSet[int]: ...
    def get_in_arc_ids(self, state_id: int) -> Sequence[int]: ...
    def get_out_arc_ids(self, state_id: int) -> Sequence[int]: ...
    def get_arc(self, arc_id: int) -> WordGraphArc: ...
    def get_state(self, state_id: int) -> WordGraphState: ...
    def is_final_state(self, state_id: int) -> bool: ...

class SmtDecoder:
    def __init__(self, model: SmtModel) -> None: ...
    @property
    def i(self) -> int: ...
    @i.setter
    def i(self, value: int) -> None: ...
    @property
    def s(self) -> int: ...
    @s.setter
    def s(self, value: int) -> None: ...
    @property
    def is_breadth_first(self) -> bool: ...
    @is_breadth_first.setter
    def is_breadth_first(self, value: bool) -> None: ...
    def translate(self, sentence: str) -> TranslationData: ...
    def translate_batch(self, sentences: Sequence[str]) -> Sequence[TranslationData]: ...
    def translate_n(self, sentence: str, n: int) -> Sequence[TranslationData]: ...
    def translate_n_batch(self, sentences: Sequence[str], n: int) -> Sequence[Sequence[TranslationData]]: ...
    def get_word_graph(self, sentence: str) -> WordGraph: ...
    def train_sentence_pair(self, source_sentence: str, target_sentence: str) -> bool: ...
    def clear(self) -> None: ...

class PhraseExtractParameters:
    monotone: bool
    max_target_phrase_length: int
    constrain_source_length: bool
    count_spurious: bool
    max_combs_in_table: int

    def __init__(self) -> None: ...

class PhraseModel:
    def __init__(self) -> None: ...
    def build(self, alignment_filename: str, parameters: PhraseExtractParameters, pseudo_ml: bool) -> bool: ...
    def print_phrase_table(self, filename: str, n: int = -1) -> bool: ...
    def clear(self) -> None: ...

class AlignmentExtractor:
    def __init__(self) -> None: ...
    def open(self, filename: str, file_format: int = 0) -> bool: ...
    def intersect(self, alignment_filename: str, output_filename: str, transpose: bool = False) -> bool: ...
    def sum(self, alignment_filename: str, output_filename: str, transpose: bool = False) -> bool: ...
    def symmetrize1(self, alignment_filename: str, output_filename: str, transpose: bool = False) -> bool: ...
    def symmetrize2(self, alignment_filename: str, output_filename: str, transpose: bool = False) -> bool: ...
    def close(self) -> None: ...

__all__ = [
    "AlignmentExtractor",
    "OnlineTrainingParameters",
    "PhraseExtractParameters",
    "PhraseModel",
    "SmtDecoder",
    "SmtModel",
    "TranslationData",
    "WordGraph",
    "WordGraphArc",
    "WordGraphState",
]



from typing import List

from pytest import approx
import numpy as np
from thot.alignment import (
    AlignmentModelType,
    AlignmentModel,
    HmmAlignmentModel,
    Ibm1AlignmentModel,
    NormalSentenceLengthModel,
    SymmetrizationHeuristic,
    SymmetrizedAligner,
)
from thot.translation import SmtModel, SmtDecoder


def test_alignment_model() -> None:
    train_src_sentences = [
        "isthay isyay ayay esttay-N .",
        "ouyay ouldshay esttay-V oftenyay .",
        "isyay isthay orkingway ?",
        "isthay ouldshay orkway-V .",
        "ityay isyay orkingway .",
        "orkway-N ancay ebay ardhay !",
        "ayay esttay-N ancay ebay ardhay .",
        "isthay isyay ayay ordway !",
    ]
    train_trg_sentences = [
        "this is a test N .",
        "you should test V often .",
        "is this working ?",
        "this should work V .",
        "it is working .",
        "work N can be hard !",
        "a test N can be hard .",
        "this is a word !",
    ]

    direct_ibm1_model = Ibm1AlignmentModel()
    _add_sentence_pairs(direct_ibm1_model, train_src_sentences, train_trg_sentences)
    _train_model(direct_ibm1_model, 2)

    direct_hmm_model = HmmAlignmentModel(direct_ibm1_model)
    direct_hmm_model.hmm_p0 = 0.1
    _train_model(direct_hmm_model, 2)

    inverse_ibm1_model = Ibm1AlignmentModel()
    _add_sentence_pairs(inverse_ibm1_model, train_trg_sentences, train_src_sentences)
    _train_model(inverse_ibm1_model, 2)

    inverse_hmm_model = HmmAlignmentModel(inverse_ibm1_model)
    inverse_hmm_model.hmm_p0 = 0.1
    _train_model(inverse_hmm_model, 2)

    aligner = SymmetrizedAligner(direct_hmm_model, inverse_hmm_model)
    aligner.heuristic = SymmetrizationHeuristic.NONE

    align_src_sentences = [
        "isthay isyay ayay esttay-N .".split(),
        "isthay isyay otnay ayay esttay-N .".split(),
        "isthay isyay ayay esttay-N ardhay .".split(),
        "".split(),
    ]
    align_trg_sentences = [
        "this is a test N .".split(),
        "this is not a test N .".split(),
        "this is a hard test N .".split(),
        "".split(),
    ]
    alignments = aligner.get_best_alignments(align_src_sentences, align_trg_sentences)
    assert len(alignments) == 4
    assert np.array_equal(alignments[0][1].to_numpy(), _create_matrix(5, [1, 2, 3, 4, 4, 5]))
    assert np.array_equal(alignments[1][1].to_numpy(), _create_matrix(6, [1, 2, 3, 4, 5, 5, 6]))
    assert np.array_equal(alignments[2][1].to_numpy(), _create_matrix(6, [1, 2, 3, 5, 4, 4, 4]))
    assert np.array_equal(alignments[3][1].to_numpy(), _create_matrix(0, []))


def test_sentence_length_model() -> None:
    model = NormalSentenceLengthModel()
    model.train_sentence_pair(10, 20)
    model.train_sentence_pair(5, 10)
    model.train_sentence_pair(7, 14)
    model.train_sentence_pair(9, 18)
    model.train_sentence_pair(11, 22)

    assert model.sentence_length_prob(8, 16) == approx(0.0966, abs=0.0001)
    assert model.sentence_length_prob(10, 20) == approx(0.0815, abs=0.0001)
    assert model.sentence_length_prob(7, 10) == approx(0.0389, abs=0.0001)


def test_smt_model() -> None:
    model = SmtModel(AlignmentModelType.FAST_ALIGN)
    params = model.online_training_parameters
    assert params.em_iters == 5
    weights = model.weights
    assert len(weights) == 8
    direct_model = model.direct_word_alignment_model
    assert direct_model.model_type == AlignmentModelType.FAST_ALIGN
    decoder = SmtDecoder(model)
    result = decoder.translate("this is a test")
    assert len(result.target) == 4


def _add_sentence_pairs(model: AlignmentModel, src_sentences: List[str], trg_sentences: List[str]) -> None:
    for src_sentence, trg_sentence in zip(src_sentences, trg_sentences):
        model.add_sentence_pair(src_sentence.split(), trg_sentence.split())


def _train_model(model: AlignmentModel, iters: int) -> None:
    model.start_training()
    for _ in range(iters):
        model.train()
    model.end_training()


def _create_matrix(src_len: int, alignment: List[int]) -> np.ndarray:
    matrix = np.full((src_len, len(alignment)), False)
    for j, i in enumerate(alignment):
        i -= 1
        if i >= 0:
            matrix[i, j] = True
    return matrix



from enum import Enum
from typing import Optional, Sequence, Tuple, overload

from ..common import WordAlignmentMatrix

class Aligner:
    def get_src_word_index(self, word: str) -> int: ...
    def get_trg_word_index(self, word: str) -> int: ...
    @overload
    def get_best_alignment(self, src_sentence: str, trg_sentence: str) -> Tuple[float, WordAlignmentMatrix]: ...
    @overload
    def get_best_alignment(
        self, src_sentence: Sequence[int], trg_sentence: Sequence[int]
    ) -> Tuple[float, WordAlignmentMatrix]: ...
    @overload
    def get_best_alignment(
        self, src_sentence: Sequence[str], trg_sentence: Sequence[str]
    ) -> Tuple[float, WordAlignmentMatrix]: ...
    @overload
    def get_best_alignments(
        self, src_sentences: Sequence[Sequence[int]], trg_sentences: Sequence[Sequence[int]]
    ) -> Sequence[Tuple[float, WordAlignmentMatrix]]: ...
    @overload
    def get_best_alignments(
        self, src_sentences: Sequence[Sequence[str]], trg_sentences: Sequence[Sequence[str]]
    ) -> Sequence[Tuple[float, WordAlignmentMatrix]]: ...

class SymmetrizationHeuristic(Enum):
    NONE = ...
    UNION = ...
    INTERSECTION = ...
    OCH = ...
    GROW = ...
    GROW_DIAG = ...
    GROW_DIAG_FINAL = ...
    GROW_DIAG_FINAL_AND = ...

class SymmetrizedAligner(Aligner):
    def __init__(self, direct_aligner: Aligner, inverse_aligner: Aligner) -> None: ...
    @property
    def heuristic(self) -> SymmetrizationHeuristic: ...
    @heuristic.setter
    def heuristic(self, value: SymmetrizationHeuristic) -> None: ...

class AlignmentModelType(Enum):
    IBM1 = ...
    IBM2 = ...
    HMM = ...
    IBM3 = ...
    IBM4 = ...
    FAST_ALIGN = ...
    INCR_IBM1 = ...
    INCR_IBM2 = ...
    INCR_HMM = ...

class AlignmentModel(Aligner):
    @property
    def model_type(self) -> AlignmentModelType: ...
    @property
    def num_sentence_pairs(self) -> int: ...
    @property
    def src_vocab_size(self) -> int: ...
    @property
    def trg_vocab_size(self) -> int: ...
    @property
    def variational_bayes(self) -> bool: ...
    @variational_bayes.setter
    def variational_bayes(self, value: bool) -> None: ...
    @property
    def max_sentence_length(self) -> int: ...
    def read_sentence_pairs(
        self, src_filename: str, trg_filename: str, counts_filename: Optional[str] = None
    ) -> None: ...
    def add_sentence_pair(
        self, src_sentence: Sequence[str], trg_sentence: Sequence[str], count: float = 1
    ) -> Tuple[int, int]: ...
    def get_sentence_pair(self, n: int) -> Tuple[Sequence[str], Sequence[str], float]: ...
    def add_src_word(self, word: str) -> int: ...
    def get_src_word(self, word_index: int) -> str: ...
    def src_word_exists(self, word: str) -> bool: ...
    def add_trg_word(self, word: str) -> int: ...
    def get_trg_word(self, word_index: int) -> str: ...
    def trg_word_exists(self, word: str) -> bool: ...
    def start_training(self) -> int: ...
    def train(self) -> None: ...
    def end_training(self) -> None: ...
    def sentence_length_log_prob(self, src_length: int, trg_length: int) -> float: ...
    def sentence_length_prob(self, src_length: int, trg_length: int) -> float: ...
    def translation_log_prob(self, src_word_index: int, trg_word_index: int) -> float: ...
    def translation_prob(self, src_word_index: int, trg_word_index: int) -> float: ...
    def get_translations(self, s: int, threshold: float = 0) -> Sequence[Tuple[int, float]]: ...
    def map_src_word_to_word_class(self, word: str, word_class: str) -> None: ...
    def map_trg_word_to_word_class(self, word: str, word_class: str) -> None: ...
    def load(self, prefix_filename: str) -> bool: ...
    def print(self, prefix_filename: str) -> bool: ...
    def clear(self) -> None: ...

class IncrAlignmentModel(AlignmentModel):
    def start_incr_training(self, sentence_pair_range: Tuple[int, int]) -> None: ...
    def incr_train(self, sentence_pair_range: Tuple[int, int]) -> None: ...
    def end_incr_training(self) -> None: ...

class Ibm1AlignmentModel(AlignmentModel):
    def __init__(self) -> None: ...

class IncrIbm1AlignmentModel(Ibm1AlignmentModel, IncrAlignmentModel):
    def __init__(self) -> None: ...

class Ibm2AlignmentModel(Ibm1AlignmentModel, AlignmentModel):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, model: Ibm1AlignmentModel) -> None: ...
    @property
    def compact_alignment_table(self) -> bool: ...
    @compact_alignment_table.setter
    def compact_alignment_table(self, value: bool) -> None: ...
    def alignment_log_prob(self, j: int, src_length: int, trg_length: int, i: int) -> float: ...
    def alignment_prob(self, j: int, src_length: int, trg_length: int, i: int) -> float: ...

class IncrIbm2AlignmentModel(Ibm2AlignmentModel, IncrAlignmentModel):
    def __init__(self) -> None: ...

class HmmAlignmentModel(Ibm1AlignmentModel, AlignmentModel):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, model: Ibm1AlignmentModel) -> None: ...
    @property
    def hmm_p0(self) -> float: ...
    @hmm_p0.setter
    def hmm_p0(self, value: float) -> None: ...
    @property
    def lexical_smoothing_factor(self) -> float: ...
    @lexical_smoothing_factor.setter
    def lexical_smoothing_factor(self, value: float) -> None: ...
    @property
    def hmm_alignment_smoothing_factor(self) -> float: ...
    @hmm_alignment_smoothing_factor.setter
    def hmm_alignment_smoothing_factor(self, value: float) -> None: ...
    def hmm_alignment_log_prob(self, prev_i: int, src_length: int, i: int) -> float: ...
    def hmm_alignment_prob(self, prev_i: int, src_length: int, i: int) -> float: ...

class IncrHmmAlignmentModel(HmmAlignmentModel, IncrAlignmentModel):
    def __init__(self) -> None: ...

class FastAlignModel(IncrAlignmentModel):
    def __init__(self) -> None: ...
    @property
    def fast_align_p0(self) -> float: ...
    @fast_align_p0.setter
    def fast_align_p0(self, value: float) -> None: ...
    def alignment_log_prob(self, j: int, src_length: int, trg_length: int, i: int) -> float: ...
    def alignment_prob(self, j: int, src_length: int, trg_length: int, i: int) -> float: ...

class Ibm3AlignmentModel(Ibm2AlignmentModel, Ibm1AlignmentModel, AlignmentModel):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, model: HmmAlignmentModel) -> None: ...
    @overload
    def __init__(self, model: Ibm2AlignmentModel) -> None: ...
    @property
    def fertility_smoothing_factor(self) -> float: ...
    @fertility_smoothing_factor.setter
    def fertility_smoothing_factor(self, value: float) -> None: ...
    @property
    def count_threshold(self) -> float: ...
    @count_threshold.setter
    def count_threshold(self, value: float) -> None: ...
    def distortion_log_prob(self, i: int, src_length: int, trg_length: int, j: int) -> float: ...
    def distortion_prob(self, i: int, src_length: int, trg_length: int, j: int) -> float: ...
    def fertility_log_prob(self, src_word_index: int, fertility: int) -> float: ...
    def fertility_prob(self, src_word_index: int, fertility: int) -> float: ...

class Ibm4AlignmentModel(Ibm3AlignmentModel, Ibm2AlignmentModel, Ibm1AlignmentModel, AlignmentModel):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, model: HmmAlignmentModel) -> None: ...
    @overload
    def __init__(self, model: Ibm3AlignmentModel) -> None: ...
    @property
    def distortion_smoothing_factor(self) -> float: ...
    @distortion_smoothing_factor.setter
    def distortion_smoothing_factor(self, value: float) -> None: ...
    def head_distortion_log_prob(self, src_word_class: int, trg_word_class: int, trg_length: int, dj: int) -> float: ...
    def head_distortion_prob(self, src_word_class: int, trg_word_class: int, trg_length: int, dj: int) -> float: ...
    def nonhead_distortion_log_prob(self, trg_word_class: int, trg_length: int, dj: int) -> float: ...
    def nonhead_distortion_prob(self, trg_word_class: int, trg_length: int, dj: int) -> float: ...

class SentenceLengthModel:
    def sentence_length_prob(self, src_length: int, trg_length: int) -> float: ...
    def sentence_length_log_prob(self, src_length: int, trg_length: int) -> float: ...
    @overload
    def train_sentence_pair(
        self, src_sentence: Sequence[str], trg_sentence: Sequence[str], count: float = 1
    ) -> None: ...
    @overload
    def train_sentence_pair(self, src_length: int, trg_length: int, count: float = 1) -> None: ...
    def load(self, filename: str) -> None: ...
    def print(self, filename: str) -> None: ...
    def clear(self) -> None: ...

class NormalSentenceLengthModel(SentenceLengthModel):
    def __init__(self) -> None: ...

__all__ = [
    "Aligner",
    "AlignmentModel",
    "AlignmentModelType",
    "FastAlignModel",
    "HmmAlignmentModel",
    "Ibm1AlignmentModel",
    "Ibm2AlignmentModel",
    "Ibm3AlignmentModel",
    "Ibm4AlignmentModel",
    "IncrAlignmentModel",
    "IncrHmmAlignmentModel",
    "IncrIbm1AlignmentModel",
    "IncrIbm2AlignmentModel",
    "NormalSentenceLengthModel",
    "SentenceLengthModel",
    "SymmetrizationHeuristic",
    "SymmetrizedAligner",
]



# -*- coding: utf-8 -*-
import os
import re
import subprocess
import sys

from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext

# Convert distutils Windows platform specifiers to CMake -A arguments
PLAT_TO_CMAKE = {
    "win32": "Win32",
    "win-amd64": "x64",
    "win-arm32": "ARM",
    "win-arm64": "ARM64",
}

# A CMakeExtension needs a sourcedir instead of a file list.
# The name must be the _single_ output extension from the CMake build.
# If you need multiple extensions, see scikit-build.
class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=""):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))

        # required for auto-detection & inclusion of auxiliary "native" libs
        if not extdir.endswith(os.path.sep):
            extdir += os.path.sep

        debug = int(os.environ.get("DEBUG", 0)) if self.debug is None else self.debug
        cfg = "Debug" if debug else "Release"

        # CMake lets you override the generator - we need to check this.
        # Can be set with Conda-Build, for example.
        cmake_generator = os.environ.get("CMAKE_GENERATOR", "")

        # Set Python_EXECUTABLE instead if you use PYBIND11_FINDPYTHON
        # EXAMPLE_VERSION_INFO shows you how to pass a value into the C++ code
        # from Python.
        cmake_args = [
            "-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={}".format(extdir),
            "-DPYTHON_EXECUTABLE={}".format(sys.executable),
            "-DCMAKE_BUILD_TYPE={}".format(cfg),  # not used on MSVC, but no harm
            "-DBUILD_SHARED_LIBRARY=OFF",
            "-DBUILD_TESTS=OFF",
        ]
        build_args = []

        if self.compiler.compiler_type != "msvc":
            # Using Ninja-build since it a) is available as a wheel and b)
            # multithreads automatically. MSVC would require all variables be
            # exported for Ninja to pick it up, which is a little tricky to do.
            # Users can override the generator with CMAKE_GENERATOR in CMake
            # 3.15+.
            if not cmake_generator:
                try:
                    import ninja  # noqa: F401

                    cmake_args += ["-GNinja"]
                except ImportError:
                    pass

        else:

            # Single config generators are handled "normally"
            single_config = any(x in cmake_generator for x in {"NMake", "Ninja"})

            # CMake allows an arch-in-generator style for backward compatibility
            contains_arch = any(x in cmake_generator for x in {"ARM", "Win64"})

            # Specify the arch if using MSVC generator, but only if it doesn't
            # contain a backward-compatibility arch spec already in the
            # generator name.
            if not single_config and not contains_arch:
                cmake_args += ["-A", PLAT_TO_CMAKE[self.plat_name]]

            # Multi-config generators have a different way to specify configs
            if not single_config:
                cmake_args += ["-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}".format(cfg.upper(), extdir)]
                build_args += ["--config", cfg]

        if sys.platform.startswith("darwin"):
            # Cross-compile support for macOS - respect ARCHFLAGS if set
            archs = re.findall(r"-arch (\S+)", os.environ.get("ARCHFLAGS", ""))
            if archs:
                cmake_args += ["-DCMAKE_OSX_ARCHITECTURES={}".format(";".join(archs))]

        # Set CMAKE_BUILD_PARALLEL_LEVEL to control the parallel build level
        # across all generators.
        if "CMAKE_BUILD_PARALLEL_LEVEL" not in os.environ:
            # self.parallel is a Python 3 only way to set parallel jobs by hand
            # using -j in the build_ext call, not supported by pip or PyPA-build.
            if hasattr(self, "parallel") and self.parallel:
                # CMake 3.12+ only.
                build_args += ["-j{}".format(self.parallel)]

        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)

        subprocess.check_call(["cmake", ext.sourcedir] + cmake_args, cwd=self.build_temp)
        subprocess.check_call(["cmake", "--build", "."] + build_args, cwd=self.build_temp)


this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# The information here can also be placed in setup.cfg - better separation of
# logic and declaration, and simpler if you include description/version in a file.
setup(
    name="sil-thot",
    version="3.4.4",
    author="SIL International",
    maintainer="Damien Daspit",
    maintainer_email="damien_daspit@sil.org",
    description="A toolkit for statistical word alignment and machine translation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="LGPL-3.0",
    platforms=["Windows", "macOS", "Linux"],
    url="https://github.com/sillsdev/thot",
    project_urls={"Repository": "https://github.com/sillsdev/thot"},
    keywords="statistical machine translation word alignment natural language processing",
    classifiers=[
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Scientific/Engineering :: Mathematics",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing",
        "Topic :: Text Processing :: Linguistic",
    ],
    ext_modules=[CMakeExtension("sil-thot")],
    cmdclass={"build_ext": CMakeBuild},
    zip_safe=False,
    extras_require={"test": ["pytest", "numpy"]},
    python_requires=">=3.7, <4.0",
    packages=["thot"],
    package_data={"thot": ["py.typed", "**/*.pyi"]},
)



from typing import Any, List, Sequence, Tuple, overload

class NGramLanguageModel:
    def __init__(self) -> None: ...
    def load(self, filename: str) -> bool: ...
    def get_sentence_log_probability(self, sentence: Sequence[str]) -> float: ...
    def clear(self) -> None: ...

class WordAlignmentMatrix:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, row_length: int, column_length: int) -> None: ...
    @property
    def row_length(self) -> int: ...
    @property
    def column_length(self) -> int: ...
    def init(self, row_length: int, column_length: int) -> None: ...
    def __getitem__(self, key: Tuple[int, int]) -> bool: ...
    def __setitem__(self, key: Tuple[int, int], value: bool) -> None: ...
    def clear(self) -> None: ...
    def reset_all(self) -> None: ...
    def set_all(self) -> None: ...
    def set(self, i: int, j: int) -> None: ...
    def put_list(self, list: List[int]) -> None: ...
    def to_list(self) -> List[int]: ...
    def transpose(self) -> None: ...
    def flip(self) -> None: ...
    def intersect(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def union(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def xor(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def add(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def subtract(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def och_symmetrize(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def priority_symmetrize(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def grow(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def grow_diag(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def grow_diag_final(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def grow_diag_final_and(self, other: WordAlignmentMatrix) -> WordAlignmentMatrix: ...
    def __eq__(self, other: object) -> bool: ...
    def to_numpy(self) -> Any: ...

__all__ = ["NGramLanguageModel", "WordAlignmentMatrix"]
